package dev.fyloz.colorrecipesexplorer.rest.account

import dev.fyloz.colorrecipesexplorer.Constants
import dev.fyloz.colorrecipesexplorer.config.annotations.PreAuthorizeAdmin
import dev.fyloz.colorrecipesexplorer.dtos.account.GroupTokenDto
import dev.fyloz.colorrecipesexplorer.dtos.account.GroupTokenSaveDto
import dev.fyloz.colorrecipesexplorer.logic.account.GroupTokenLogic
import dev.fyloz.colorrecipesexplorer.logic.account.JwtLogic
import dev.fyloz.colorrecipesexplorer.rest.created
import dev.fyloz.colorrecipesexplorer.rest.noContent
import dev.fyloz.colorrecipesexplorer.rest.ok
import dev.fyloz.colorrecipesexplorer.utils.addCookie
import org.springframework.context.annotation.Profile
import org.springframework.web.bind.annotation.*
import javax.servlet.http.HttpServletResponse
import javax.validation.Valid

@RestController
@RequestMapping(Constants.ControllerPaths.GROUP_TOKEN)
@PreAuthorizeAdmin
@Profile("!emergency")
class GroupTokenController(
    private val groupTokenLogic: GroupTokenLogic,
    private val jwtLogic: JwtLogic
) {
    @GetMapping
    fun getAll() = ok(groupTokenLogic.getAll())

    @GetMapping("{id}")
    fun getById(@PathVariable id: String) = ok(groupTokenLogic.getById(id))

    // TODO Remove when group tokens will be fully implemented
    @Deprecated("Only use for testing purposes")
    @GetMapping("{id}/cookie")
    fun addCookieForId(@PathVariable id: String, response: HttpServletResponse) {
        val groupToken = groupTokenLogic.getById(id)
        addGroupTokenCookie(response, groupToken)
    }

    @PostMapping
    fun save(@RequestBody @Valid dto: GroupTokenSaveDto, response: HttpServletResponse) =
        with(groupTokenLogic.save(dto)) {
            addGroupTokenCookie(response, this)
            created(Constants.ControllerPaths.GROUP_TOKEN, this, this.id)
        }

    @DeleteMapping("{id}")
    fun deleteById(@PathVariable id: String) = noContent {
        groupTokenLogic.deleteById(id)
    }

    private fun addGroupTokenCookie(response: HttpServletResponse, groupToken: GroupTokenDto) {
        val jwt = jwtLogic.buildGroupTokenIdJwt(groupToken.id)
        val bearer = Constants.BEARER_PREFIX + jwt

        response.addCookie(Constants.CookieNames.GROUP_TOKEN, bearer) {
            httpOnly = GROUP_TOKEN_COOKIE_HTTP_ONLY
            sameSite = GROUP_TOKEN_COOKIE_SAME_SITE
            secure = !Constants.DEBUG_MODE
            maxAge = null   // This cookie should never expire
            path = GROUP_TOKEN_COOKIE_PATH
        }
    }

    companion object {
        private const val GROUP_TOKEN_COOKIE_HTTP_ONLY = true
        private const val GROUP_TOKEN_COOKIE_SAME_SITE = true
        private const val GROUP_TOKEN_COOKIE_PATH = Constants.ControllerPaths.GROUP_LOGIN
    }
}
